#! /usr/bin/perl

#$Id: autogen.pl,v 1.18 2002/06/17 18:57:10 itz Exp $

use POSIX;
use Text::Template;
use FileHandle;
use String::Escape qw(printable);

$progname = "gim";
$whatis = "Distributes gii mouse events to programs on text mode virtual terminals.";
$email = q(itz@speakeasy.org);

@opts =
    (
     { name => "age_limit", flag => "a", arg => 1,
       docstring => "Disable pasting selection after !NAME! seconds" },
     { name => "buttons", flag => "b", arg => 1,
       docstring => "Reinterpret gii button numbers according to !NAME!" },
     { name => "device", flag => "d", arg => 1,
       docstring => "Mouse attached to !NAME! (can be a symlink)" },
     { name => "file", flag => "f", arg => 1,
       docstring => "Listen on Unix address !NAME!" },
     { name => "foreground", flag => "F", arg => 0, 
       docstring => "Do not drop into background; log events on console" },
     { name => "help", flag => "h", arg => 0,
       docstring => "Print help information successfully" },
     { name => "interval", flag => "i", arg => 1, 
       docstring => "Allow !NAME! milliseconds between double-clicks" },
     { name => "ignore", flag => "I", arg => 1,
       docstring => "Ignore terminals listed in !NAME! (e.g. your xdm vt)" },
     { name => "protocol", flag => "p", arg => 1,
       docstring => "Mouse uses !NAME! (see libgii file `inputs.txt')" },
     { name => "pointer", flag => "P", arg => 0,
       docstring => "Draw mouse pointer during selection" },
     { name => "repeater", flag => "R", arg => 1,
       docstring => "Use !NAME! as repeater DLL" },
     { name => "scale_down", flag => "s", arg => 1,
       docstring => "Scale coordinates in gii events down by !NAME!" },
     { name => "scale_up", flag => "S", arg => 1,
       docstring => "Scale coordinates in gii events up by !NAME!" },
     { name => "threshold", flag => "t", arg => 1,
       docstring => "Ignore gii mouse movements smaller than !NAME!" },
     { name => "verbose", flag => "v", arg => 0,
       docstring => "Verbosely log all events" },
     { name => "version", flag => "V", arg => 0,
       docstring => "Print version information successfully" },
     { name => "wchars", flag => "w", arg => 1,
       docstring => "!NAME! is a list of characters for selection by words" },
     );

$gimev_progname = "gimev";
$gimev_whatis = "A simple example gim client.";

@gimev_opts =
    (
     { name => "device", flag => "d", arg => 1,
       docstring => "Mouse attached to !NAME! (can be a symlink)" },
     { name => "event_mask", flag => "e", arg => 1,
       docstring => "Ignore events not in !NAME!" },
     { name => "help", flag => "h", arg => 0,
       docstring => "Print help information successfully" },
     { name => "max_mod", flag => "M", arg => 1,
       docstring => "Ignore events with keyboard modifiers not in !NAME!" },
     { name => "min_mod", flag => "m", arg => 1,
       docstring => "Ignore events without keyboard modifiers in !NAME!" },
     { name => "pass_mask", flag => "p", arg => 1,
       docstring => "Pass events in !NAME! to root gim clients" },
     { name => "pointer", flag => "P", arg => 0,
       docstring => "Draw mouse pointer during clicks and drags" },
     { name => "terminal", flag => "t", arg => 1,
       docstring => "Listen for mouse events on !NAME!" },
     );

@gimev_cmds =
    (
     { name => "devinfo", arg => 0, result => 1,
       docstring => "Print mouse device information (copied straight from libgii)" },
     { name => "disable_paste", arg => 0, result => 0,
       docstring => "Tell server to disable pasting until a fresh selection is made" },
     { name => "handle", arg => 1, result => 0,
       docstring => "Change handled event mask to ARG" },
     { name => "maxmod", arg => 1, result => 0,
       docstring => "Change maximum modifier mask to ARG" },
     { name => "minmod", arg => 1, result => 0,
       docstring => "Change minimum modifier mask to ARG" },
     { name => "pass", arg => 1, result => 0,
       docstring => "Change passed event mask to ARG" },
     { name => "start", arg => 0, result => 0,
       docstring => "Start receiving mouse events" },
     { name => "stop", arg => 0, result => 0,
       docstring => "Stop receiving mouse events" },
     { name => "servinfo", arg => 0, result => 1,
       docstring => "Print gim server information, including protocol version" },
     );

$t_copy = new Text::Template(TYPE => FILE, SOURCE => 'COPYING.pl', DELIMITERS => ['{*','*}']);
$copy = $t_copy->fill_in();

foreach $output (@ARGV) {
    $template = new Text::Template(TYPE => FILE, SOURCE => "$output.pl", DELIMITERS => ['{*','*}']);
    $outfile = new FileHandle($output,"w");
    $outfile or die "$outfile: $!\n";
    $template->fill_in(OUTPUT => $outfile);
    $outfile->close();
}
