/*
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This code is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this code; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.  */

/* Copyright (C) 2000 Ian Zimmerman <itz@speakeasy.org> */

#ifndef INCLUDE_GIM_EVENT_H
#define INCLUDE_GIM_EVENT_H

/* $Id: gim_event.h,v 1.10 2002/08/17 18:41:04 itz Exp $ */

#include "gim_errno.h"

/* START PUBLIC */

#include <sys/time.h>
#include <stdio.h>

enum {
  gim_move = 1,
  gim_drag,
  gim_down,
  gim_up,
  gim_dragged_click,
  gim_single,
  gim_double,
  gim_triple,
  gim_enter,
  gim_leave,
  gim_last_ev
};

#define EVDEF(x) x ## _ev = (1 << ((x) - 1))

enum  gim_event_kind_t {
  EVDEF(gim_move),
  EVDEF(gim_drag),
  gim_position_ev = (gim_move_ev | gim_drag_ev),
  EVDEF(gim_down),
  EVDEF(gim_up),
  gim_button_ev = (gim_down_ev | gim_up_ev),
  gim_raw_ev = (gim_position_ev | gim_button_ev),
  EVDEF(gim_dragged_click),
  EVDEF(gim_single),
  EVDEF(gim_double),
  EVDEF(gim_triple),
  gim_multi_ev = (gim_single_ev | gim_double_ev | gim_triple_ev),
  EVDEF(gim_enter),
  EVDEF(gim_leave),
  gim_synth_ev = (gim_enter_ev | gim_leave_ev),
  EVDEF(gim_last_ev)
};

enum {
  gim_top = 1,
  gim_left,
  gim_bottom,
  gim_right,
  gim_last_mg
};

#define MGDEF(x) x ## _mg = (1 << ((x) - 1))  

enum gim_margin_t {
  MGDEF(gim_top),
  MGDEF(gim_left),
  gim_topleft_mg = (gim_top_mg | gim_left_mg),
  MGDEF(gim_bottom),
  gim_bottomleft_mg = (gim_bottom_mg | gim_left_mg),
  MGDEF(gim_right),
  gim_topright_mg = (gim_top_mg | gim_right_mg),
  gim_bottomright_mg = (gim_bottom_mg | gim_right_mg),
  gim_vertical_mg = (gim_top_mg | gim_bottom_mg),
  gim_horizontal_mg = (gim_left_mg | gim_right_mg),
  gim_none_mg = 0,
  MGDEF(gim_last_mg)
};

struct gim_event_t {
  enum gim_event_kind_t kind;
  int x;                        /* absolute coordinates */
  int y;
  int z;
  unsigned state;               /* mask of first 32 pressed buttons */
  unsigned modifiers;
  struct timeval time;          /* timestamp */
  int vt;
  union {
    struct {
      int x;
      int y;
      int z;
      enum gim_margin_t margin;
    } d;                        /* only for moves and drags */
    struct {
      int which;
      int repeat;
    } b;                        /* only for downs and ups */
  } u;
};

int
gim_get_event (int fd, struct gim_event_t* p_ev);

#define GIM_GET_EVENT_SUCCESS GIM_SUCCESS
#define GIM_GET_EVENT_ERR_PROTOCOL -300
#define GIM_GET_EVENT_ERR_UNKNOWN GIM_ERR_UNKNOWN

struct gim_handler_data_t {
  int morekeys;
  int is_fake;
  void* extra_data;
};

typedef int (*gim_handler)(struct gim_event_t*, struct gim_handler_data_t*);

/* this should really not be public, but it is needed for the wgetch macro below */
typedef int (*gim_getcfun_t)(void* p);

int
gim_event_select (int fd, gim_handler h, struct gim_handler_data_t* hdata,
                  gim_getcfun_t p, void* arg);

int 
gim_getchar (int fd, gim_handler h, struct gim_handler_data_t* hdata);

/* in this way we avoid the dependency on curses, hee hee */
#define gim_wgetch(fd, h, hdata, w) (gim_event_select((fd), (h), (hdata), (int (*)(void*))wgetch, (w)))

struct gim_region_t {
  int x_min, x_max, y_min, y_max;
  unsigned min_mod, max_mod;
  enum gim_event_kind_t handle;
  gim_handler handler_fun;
  void* extra_data;
  int active;
  struct gim_region_t* prev;
  struct gim_region_t* next;
};

void
gim_init_region (struct gim_region_t* p_reg);

struct gim_region_t*
gim_alloc_region (void);

void
gim_put_region (struct gim_region_t* p_reg, struct gim_region_t* after);

int
gim_region_handler (struct gim_event_t* p_ev, struct gim_handler_data_t* hdata);

/* END PUBLIC */

void
gim_get_event_internal (int fd, struct gim_event_t* p_ev);

#endif /* INCLUDE_GIM_EVENT_H */
